#include <stdio.h>
#include <iostream>
#include <stdlib.h> 
#include <unistd.h> 
#include <fcntl.h>
#include <sys/ioctl.h>

#include <qpushbutton.h>
#include <qkeycode.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qregexp.h>
#include <kstatusbar.h>
#include <qdir.h>

#include <kmainwindow.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kurl.h>

#include <kprocess.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xmu/WinUtil.h>	/* for XmuClientWindow() */


/* RUNNING MODES */
#define KJABATA_NORMAL_MODE   0  /* Normal mode, playing and recording */
#define KJABATA_RECORD_MODE   2  /* Manual recording mode */
#define KJABATA_PLAY_MODE     3  /* Playing only mode, no recording enabled */
#define KJABATA_CHECKBOX_MODE 4  /* Only recordings with checkbox "ON AIR" */

#include "kjabata.moc"
#define _(TEXT) i18n(TEXT)

#define KJABATAVERSION "0.9.5"


/* STATUS BAR IDs */
#define STAT_CLOCK       		1
#define STAT_BLKCOUNTDOWN		2
#define STAT_SEPARATOR			3
#define STAT_SNDCOUNTDOWN		4

/* STATES */
#define PLAYING			1
#define RECORDING		2
#define STOPINGPLAY		3
#define STOPINGRECORD   4
#define STOPED			5

/* TIMES */
#define CLOCKMILISECONDS    900  /* The main event loop tick */
#define WAITTORECORDCLKS    10

#define ERRORSECONDS    45    /* While playing */
#define FORWARDSECONDS 180    /* When we are stopped or recording */

#define MINSECSLEFTAUTOREC 900 /* While in recording mode, minimum 
                                  number of seconds left to start to record */

#define MAXSLOTSECS	1860       /* 1/2 hour + 60 seconds */
#define MAXPROGRAMDELAY 3600   /* Time that a program can extend its expected 
                                  end time. */


#define SAMPLINGFREQUENCY 16000

/* GUI appearance */
#define DEFAULTWIDTH  640
#define DEFAULTHEIGHT 480

#define SPACE	4

#define BUTTONWIDTH   74
#define BUTTONHEIGHT  64

#define CSOUNDROWS    5
#define BLOCKROWS     5


KApplication *globalKapp;
KIconLoader  *globalKIL;
KLocale      *globalKlocale;


const char * replaceAcutes (const char *src)
{
    static QString str;
    str = src;
    str.replace (QRegExp ("&ntilde;"), "ny");
    str.replace (QRegExp ("&Ntilde;"), "NY");
    str.replace (QRegExp ("&"),"");
    str.replace (QRegExp ("acute;"), "");
    return str.data ();
}



/* TODO: Remove main from here, put main code somewhere else
         different from class code. */

/* TODO: Make a kjabata_gui class, where all GUI code should go
         we must sepparate GUI code from the rest of kjabata code! */

int main (int argC, char *argV[])
{
    KontinuidadJabata 	*kontinudadJabata;
    JabatoSchedule 	    sch;
    char 		        *dev=NULL;
    int		            kjabatamode;
    char                        *output=NULL;
    KCmdLineArgs        *args;
    QCString            OptionArg;

    /* TODO: We need to add a --verbose option and remove all the
       ugly traces when not set. */

    static KCmdLineOptions options[] = { { "d ",
                                           I18N_NOOP("sound device"), 
                                           "/dev/dsp" },
                                         { "i ",
                                           I18N_NOOP("input device"),
                                           "/dev/dsp" },
                                         { "m ",
                                           I18N_NOOP("running mode"),
                                           "0" },
                                         { "o ",
                                           I18N_NOOP("output device"),
                                           "/dev/dsp" },
                                         {  0  , 0, 0 } };

    KCmdLineArgs::init (argC, 
                        argV, 
                        "kjabata", 
                        "kjabata", 
                        "Kontinuidad Jabata",
                        KJABATAVERSION);

    KCmdLineArgs::addCmdLineOptions (options);
    globalKapp = new KApplication ();

    args = KCmdLineArgs::parsedArgs ();
    kjabatamode = KJABATA_NORMAL_MODE;
    if (args->isSet ("d"))
    {
        OptionArg = args->getOption ("d");
        output = dev = OptionArg.data ();
        cout << "WARNING: The option -d is obsolete and will be removed";
    } else {
        if (args->isSet ("i"))
        {
            OptionArg = args->getOption ("i");
            dev = OptionArg.data ();
        } else
            dev = "/dev/dsp";
        if (args->isSet ("o"))
        {
            OptionArg = args->getOption ("o");
            output = OptionArg.data ();
        } else
            output = "/dev/dsp";
    }

    if (args->isSet ("m"))
    {
        OptionArg = args->getOption ("m");
        kjabatamode=atoi(OptionArg.data ()); 

    }

    cout <<  "Radio Jabato's KJabata " << KJABATAVERSION << 
	         " running " << kjabatamode << " Mode "; 
    if (dev)
      cout << "on " << dev;    
    if (output)
        cout << " Playing on " << output;
    cout << "\nBy kjabata-tech@lists.sourceforge.net\n";

    if (!sch.initSchedule ()) 
        return -1;

    globalKIL      = KGlobal::iconLoader ();
    globalKlocale  = KGlobal::locale ();
    kontinudadJabata = new KontinuidadJabata (&sch, dev, output, kjabatamode);

    return globalKapp->exec ();
}


KontinuidadJabata::KontinuidadJabata (JabatoSchedule *jSch, 
                                      char *dev, char *output, 
                                      int kjmode) : KMainWindow (0, "main")
{
    QFileInfo file;

    if (dev) 
        sprintf (text,"%s",dev);
    else
        sprintf (text,"dev/dsp");
  
    sprintf (startRecScript, "%s/%s/startRecording.sh", CFGDIR, text);
    file.setFile (startRecScript);

    if (!file.exists ()) 
    {
        fprintf(stderr, "kontinuidadJabata: %s NOT FOUND\n", startRecScript);
        startRecScript[0]=0;
    }

    sprintf (holdRecScript, "%s/%s/holdRecording.sh", CFGDIR, text);
    file.setFile (holdRecScript);

    if (!file.exists ()) 
    {
        fprintf(stderr, "kontinuidadJabata: %s NOT FOUND\n", holdRecScript);
        holdRecScript[0]=0;
    }

    sprintf (restartRecScript, "%s/%s/restartRecording.sh", CFGDIR, text);
    file.setFile (restartRecScript);

    if (!file.exists ()) 
    {
        fprintf(stderr, "kontinuidadJabata: %s NOT FOUND\n", restartRecScript);
        restartRecScript[0]=0;
    }
  
    sprintf (finishRecScript, "%s/%s/finishRecording.sh", CFGDIR, text);
    file.setFile (finishRecScript);

    if(!file.exists()) 
    {
        fprintf(stderr, "kontinuidadJabata: %s NOT FOUND\n",finishRecScript);
        finishRecScript[0]=0;
    }
  
    kjabatamode = kjmode;
    status      = STOPED;

    changingBlock = false;
    autoRecording = false; /* if "autorecording" is enabled in the .jsf file,
                              autoRecording will be set to true whenever 
                              record () is called from code. */
    autoRecorded = false;
    recordPending = false;
    recorderTries = 0;
    clksWaitingToRecord = 0;
    schedule = jSch;

    if (schedule->autoRecMode () && 
        (kjabatamode == KJABATA_NORMAL_MODE || 
         kjabatamode == KJABATA_RECORD_MODE || 
         kjabatamode == KJABATA_CHECKBOX_MODE)) 

        fprintf(stderr, "kontinuidadJabata: autoRecording mode ON \n");
    else
        fprintf(stderr, "kontinuidadJabata: autoRecording mode OFF \n");

    filefeeder = new JabatoFileFeeder(schedule);
    currentSound = new JabatoSound();
    nextSound = new JabatoSound();

    mp3player = new MP3Player (output);
    connect (mp3player, 
             SIGNAL (finished (MP3Player*)),
	         this,
             SLOT (playbackFinished (MP3Player*)));

    if (schedule->samplingFreq () > 0) 
    {
        mp3recorder = new MP3Recorder (schedule->samplingFreq (), 2, 2, dev);
        minFreeDiskInBytes = schedule->samplingFreq () * 10;
    }
    else 
    {
        mp3recorder = new MP3Recorder (SAMPLINGFREQUENCY, 2, 2);
        minFreeDiskInBytes = SAMPLINGFREQUENCY * 10;
    }

    connect (mp3recorder, 
             SIGNAL (started (MP3Recorder*)),
	         this,
             SLOT (recordingStarted (MP3Recorder*)));

    connect (mp3recorder,
             SIGNAL (hold (MP3Recorder*)),
	         this,
             SLOT (recordingHold(MP3Recorder*)));

    connect (mp3recorder,
             SIGNAL (restarted (MP3Recorder*)),
             this,
             SLOT (recordingRestarted(MP3Recorder*)));

    connect (mp3recorder,
             SIGNAL (finished (MP3Recorder*)),
             this,
             SLOT (recordingFinished (MP3Recorder*)));

    PopulateGUI ();

    lookforNextSound ();
    refreshNextSound ();
    refreshCurrentBlock ();
    refreshCurrentSound ();
      					
    /* The following line starts the heart of Kjabata, it needs to pump every 
       CLOCKMILISECONDS to check for new events that change the complex state 
       machine.
       It would be desirable do this without involving a GUI. Is it possible
       to keep the KDE infraestructure without a GUI? */ 
    QTimer::singleShot (CLOCKMILISECONDS, this, SLOT (timeout ()));

    shellProcess = new KShellProcess ();
    freeDiskProcess = new KShellProcess ();
    connect (freeDiskProcess, 
             SIGNAL (receivedStdout (KProcess *, char *, int)),
             this, 
             SLOT (freeDiskOutput (KProcess *, char *, int)));
 
    if (kjabatamode != KJABATA_RECORD_MODE && 
        kjabatamode != KJABATA_CHECKBOX_MODE)
        playORstop();
}


KontinuidadJabata::~KontinuidadJabata()
{
  DEBUG(fprintf(stderr, "~KontinuidadJabata\n"));
/*
  delete recordButton;
  delete forwardButton;
  delete mixerButton;
  delete stopNowButton;

  delete qgLevel;
  delete qgFFT;
  
  delete statusline;

  delete lbCSndName;
  delete lbCSndTime;
  delete lbCSndDir;
  delete lbCSndSubdir;
  delete lbCSndPath;
  delete cbBlocks;
  delete lbBlkSonidos;
  delete lbBlkBegin;
  delete lbBlkEnd;
  delete cbSlots;
  delete lbBlkBeganAt;
  delete lbBlkllFinishAt;
  delete lbBeganArrow;
  delete lbFinishArrow;
  delete lbSndName;
  delete lbSndTime;
  delete lbSndDir;
  delete lbSndSubdir;
  delete lbSndPath;
  
  delete qgTop;
  delete qgCSound;
  delete qgSound;
  delete qgBlock;


  delete shellProcess;
  delete freeDiskProcess;
  
  delete mp3player;
  delete mp3recorder;
  
  delete filefeeder;

  delete currentSound;
  delete nextSound;
*/
}


void KontinuidadJabata::PopulateGUI ()
{
    unsigned int i, x, y, w;
    QSize size;
    bool must_restore;

    globalKapp->setMainWidget (this);
    move (0,0);

    const QPixmap qMainIcon ("kontinuidadJabata.png");
    QWidget::setIcon (qMainIcon);

    setCaption (globalKapp->makeStdCaption (schedule->getStationName ()),
                                            false);

    qgTop = new QGroupBox (this);
    qgTop->setFrameStyle (0);
    qgSound = new QGroupBox (qgTop);
    qgBlock = new QGroupBox (qgTop);
    qgCSound = new QGroupBox (qgTop);
  
    /* Buttons */
    KIconLoader *loader = KGlobal::iconLoader ();

    recordButton = new QPushButton (qgSound);
    forwardButton = new QPushButton (qgSound);
 
    /* icons */ 
    playIcon = loader->loadIcon ("mforward.xpm", KIcon::User);
    forwardButton->setPixmap (playIcon);
    playIcon = loader->loadIcon ("mplay.xpm", KIcon::User);
    stopIcon = loader->loadIcon ("mstop.xpm", KIcon::User);
    recIcon = loader->loadIcon ("mrecord.xpm", KIcon::User);
    recStopIcon = loader->loadIcon ("mrecstop.xpm", KIcon::User);
  
    recordButton->setPixmap (recIcon);
  
    mixerButton = new QPushButton(qgTop);

    connect (forwardButton, SIGNAL(clicked()), this, SLOT(forwardSound()));
    connect (recordButton, SIGNAL(clicked()), this, SLOT(record()));
    connect (mixerButton, SIGNAL(clicked()), this, SLOT(runMixer()));

    QWidget  *tab[2][15];
    int       tabw[2],tabh[15];


    /* NEXT SOUND FRAME -------------------------------------------------- */
    qgSound->setTitle (_("SIGUIENTE SONIDO"));
    tab[0][0] = new QLabel (_("- Nombre:"), qgSound, "lbTSndName");
    tab[0][1] = new QLabel (_("- Duracion:"), qgSound, "lbTSndDuracion");
    tab[0][2] = new QLabel (_("- Carpeta:"), qgSound, "lbTSndDir");
    tab[0][3] = new QLabel (_("- Subcarpeta:"), qgSound, "lbTSndSubdir");
    tab[0][4] = new QLabel (_("- Ruta:"), qgSound, "lbTSndPath");

    tab[1][0] = lbSndName = new QLabel ("", qgSound, "lbSndName");
    tab[1][1] = lbSndTime = new QLabel (_("0:00:00"), qgSound, "lbSndTime");
    tab[1][2] = lbSndDir = new QLabel (_(""), qgSound, "lbSndDir");
    tab[1][3] = lbSndSubdir = new QLabel ("", qgSound, "lbSndSubdir");
    tab[1][4] = lbSndPath = new QLabel (_(""), qgSound, "lbSndPath");


    /* CURRENT/NEXT BLOCK FRAME ------------------------------------------ */
    qgBlock->setTitle (_("BLOQUE ACTUAL"));
    tab[0][5] = new QLabel (_("- Nombre:"), qgBlock, "lbTBlkName");
    tab[0][6] = new QLabel (_("- Comienzo:"), qgBlock, "lbTBlkBeginAt");
    tab[0][7] = new QLabel (_("- Final:"), qgBlock, "lbTBlkEndsAt");
    tab[0][8] = new QLabel (_("- Sonidos:"), qgBlock, "lbTBlkSonidos");
    tab[0][9] = new QLabel (_("- Cuñas:"), qgBlock, "lbTBlkOrden");

    cbBlocks = new QComboBox (FALSE, qgBlock, "cbBlocks");
    cbBlocks->setMinimumSize(QSize (400, 18)); 

    for (i=0; i<schedule->blocksCount (); i++) 
       cbBlocks->insertItem (replaceAcutes (schedule->getBlockName ((uint)i)));

    connect (cbBlocks,
             SIGNAL (activated(int)),
             this,
             SLOT (blockChanged(int)));

    cbSlots = new QComboBox (FALSE, qgBlock, "cbSlots");
    cbSlots->setMinimumSize(QSize (400, 18)); 
    connect (cbSlots, SIGNAL (activated (int)), this, SLOT (slotChanged(int)));

    tab[1][5] = cbBlocks;
    tab[1][6] = lbBlkBegin = new QLabel (_(""), qgBlock, "lbBlkBeginAt");
    tab[1][7] = lbBlkEnd = new QLabel (_(""), qgBlock, "lbBlkEndsAt");
    tab[1][8] = lbBlkSonidos = new QLabel (_(""), qgBlock, "lbBlkSonidos");
    tab[1][9] = cbSlots;

    lbBlkBeganAt = new QLabel (_("00:00:00"), qgBlock, "lbBlkBeganAt");
    lbBlkllFinishAt = new QLabel (_("00:00:00"), qgBlock, "lbBlkFinishAt");
    lbBeganArrow = new QLabel (
                          _(">------------ comenzara a las... ----------->"),
                          qgBlock, 
                          "lbBeganArrow");
    lbFinishArrow = new QLabel (
                          _(">------------- terminara a las... ------------>"),
                          qgBlock, 
                          "lbFinishArrow");


    /* CURRENT SOUND FRAME ----------------------------------------------- */
    qgCSound->setTitle (_("SONIDO ACTUAL"));
    tab[0][10] = new QLabel (_("- Nombre:"), qgCSound, "lbTCSndName");
    tab[0][11] = new QLabel (_("- Duracion:"), qgCSound, "lbTCSndDuracion");
    tab[0][12] = new QLabel (_("- Carpeta:"), qgCSound, "lbTCSndDir");
    tab[0][13] = new QLabel (_("- Subcarpeta:"), qgCSound, "lbTCSndSubdir");
    tab[0][14] = new QLabel (_("- Ruta:"), qgCSound, "lbTCSndPath");

    tab[1][10] = lbCSndName = new QLabel ("", qgCSound, "lbCSndName");
    tab[1][11] = lbCSndTime = new QLabel (_("0:00:00"), qgCSound, "lbCSndTime");
    tab[1][12] = lbCSndDir = new QLabel (_(""), qgCSound, "lbCSndDir");
    tab[1][13] = lbCSndSubdir = new QLabel ("", qgCSound, "lbCSndSubdir");
    tab[1][14] = lbCSndPath = new QLabel (_(""), qgCSound, "lbCSndPath");

    stopNowButton = new QPushButton (qgCSound);
    stopNowButton->setPixmap (stopIcon);
    stopNowButton->hide ();
    connect (stopNowButton, SIGNAL (clicked ()), this, SLOT (stopNow ()));

    
    /* Arrange widgets */
    for (i = 0; i < sizeof (tabw)/sizeof (int); i++) 
        tabw[i] = 0;

    for (i = 0; i < sizeof (tabh)/sizeof (int); i++)
         tabh[i] = 0;

    for (y = 0; y < sizeof (tabh)/sizeof (int); y++) 
    {
        for (x = 0; x < sizeof (tabw)/sizeof (int); x++) 
        {
            size = tab[x][y]->sizeHint ();
            if (size.isValid ())	
            {
                if (size.width () > tabw[x])	
                    tabw[x] = size.width ();

                if (size.height () > tabh[y]) 
                    tabh[y] = size.height ();
            }
        } 
    }
	
    for (i = 0; i < sizeof (tabw)/sizeof (int); i++)
        tabw[i] += SPACE;

    csndGrpHeight = 0;
    blkGrpHeight = 0;
    sndGrpHeight = 0;

    for (y = 0, sndGrpHeight = 4+3*SPACE; 
                y < sizeof (tabh)/sizeof (int); 
                y++, sndGrpHeight+=SPACE)
    {
        if(y == CSOUNDROWS)
        {
            csndGrpHeight = sndGrpHeight + SPACE;
            sndGrpHeight = 4+3*SPACE;
        }

        if(y == (CSOUNDROWS+BLOCKROWS)) 
        {
            blkGrpHeight = sndGrpHeight + SPACE;
            sndGrpHeight = 4+3*SPACE;
        }

        for (x = 0, w = 2*SPACE; x < sizeof (tabw)/sizeof (int); x++, w+=SPACE)
        {
            tab[x][y]->setGeometry (w, sndGrpHeight, tabw[x], tabh[y]);
            w += tabw[x];
        }

        sndGrpHeight += tabh[y];
    }

    sndGrpHeight += SPACE;

    forwardButton->setGeometry ((int) ((tabw[0]+tabw[1])*1.1),
                                SPACE*8+3,
                                SPACE*6,
                                SPACE*5);

    recordButton->setGeometry ((int) ((tabw[0]+tabw[1])*1.1 + 40),
                               SPACE*8+3,
                               SPACE*6,
                               SPACE*5);

    stopNowButton->setGeometry ((int) ((tabw[0]+tabw[1])*1.1),
                                SPACE*8+3,
                                SPACE*6,
                                SPACE*5);

    if (kjabatamode == KJABATA_CHECKBOX_MODE)
    { 
        onAirCheck = new QCheckBox (_("AL AIRE"), qgTop);
        onAirCheck->setChecked (true);
        onAirCheck->setGeometry (BUTTONWIDTH*2 + SPACE*5+3,
                                 BUTTONHEIGHT+SPACE*3,
             	                 100,
                                 10);
    }

    lbBeganArrow->setGeometry (lbBlkBegin->x() + 48,
                               lbBlkBegin->y(),
                               40,
                               10);
    lbBeganArrow->setMinimumSize (lbBeganArrow->sizeHint ());

    lbBlkBeganAt->setGeometry (
                          lbBeganArrow->x () + lbBeganArrow->width () + SPACE*3,
                          lbBlkBegin->y (), 
                          40,
                          10);

    lbBlkBeganAt->setMinimumSize (lbBlkBeganAt->sizeHint ());
    lbFinishArrow->setGeometry (lbBlkEnd->x () + 48, lbBlkEnd->y (), 70, 10);
    lbFinishArrow->setMinimumSize (lbFinishArrow->sizeHint ());

    lbBlkllFinishAt->setGeometry (
          lbBeganArrow->x () + lbBeganArrow->width () + SPACE*3, lbBlkEnd->y (),
          40,
          10);

    lbBlkllFinishAt->setMinimumSize (lbBlkllFinishAt->sizeHint ());


    /* SOUND LEVEL ------------------------------------------------------- */
    qgLevel = new QFrame (qgCSound);
    qgLevel->setLineWidth (1);
    qgLevel->setFrameStyle (QFrame::Box | QFrame::Raised);

    qgLevel->setGeometry (330,
                          20,
                          BUTTONWIDTH*2,
                          BUTTONHEIGHT+1);


    mp3recorder->setLevelFrame (qgLevel);
    qgLevel->hide ();

    mixerButton->setGeometry (BUTTONWIDTH*5 + SPACE*9,
                              SPACE,
                              SPACE,
                              BUTTONHEIGHT+1);
    mixerButton->hide ();

    qgFFT = new QFrame (qgCSound);
    qgFFT->setLineWidth (1);
    qgFFT->setFrameStyle (QFrame::Box | QFrame::Raised);

    qgFFT->setGeometry (330 + BUTTONWIDTH*2,
                        20,
                        BUTTONWIDTH*2,
                        BUTTONHEIGHT+1);

    mp3recorder->setFFTFrame (qgFFT);
    qgFFT->hide ();

    qgSound->setLineWidth (1);
    qgSound->setFrameStyle (QFrame::Box | QFrame::Sunken);

    qgBlock->setLineWidth (1);
    qgBlock->setFrameStyle (QFrame::Box | QFrame::Sunken);

    qgCSound->setLineWidth (1);
    qgCSound->setFrameStyle (QFrame::Box | QFrame::Sunken);

    qgSound->setGeometry (SPACE,
                          20,
                          DEFAULTWIDTH-SPACE*2,
                          csndGrpHeight);

    qgBlock->setGeometry (SPACE,
                          40 + csndGrpHeight,
                          DEFAULTWIDTH-SPACE*2,
                          blkGrpHeight);

    qgCSound->setGeometry (SPACE,
                           60 + blkGrpHeight + csndGrpHeight,
                           DEFAULTWIDTH-SPACE*2,
                           sndGrpHeight);

    setMinimumSize (DEFAULTWIDTH, DEFAULTHEIGHT);
    create_statusline();

    /* session management */
    if (globalKapp->isRestored ()) 
    {
        for (i = 1; canBeRestored (i); i++)
            if (0 == strcmp (classNameOfToplevel (i), "KontinuidadJabata"))
                break;

        if (!canBeRestored (i))
            must_restore = false;
        else
            must_restore = true;
    }
    else
    {
        must_restore = false;
    }

    if (must_restore)
    {
        restore (i);
    }
    else 
    {
        resize (DEFAULTWIDTH,DEFAULTHEIGHT);
        show ();
    }

}


void KontinuidadJabata::resizeEvent (QResizeEvent *event)
{
    /* TODO: Disable resizing of the window. There really is no need
             at all to resize this kind of application!!! */

    int w = this->width ();
    int h = this->height ();

    qgTop->setGeometry (0, 0, w, h);

    statusline->setGeometry (6, h-20, w-6, 18);

 /*   qgSound->setGeometry (SPACE,
                          h-csndGrpHeight-sndGrpHeight-blkGrpHeight,
                          w-SPACE*2,
                          csndGrpHeight);

    qgBlock->setGeometry (SPACE,
                          h-csndGrpHeight-blkGrpHeight,
                          w-SPACE*2,
                          blkGrpHeight);

    qgCSound->setGeometry (SPACE,
                           h-csndGrpHeight,
                           w-SPACE*2,
                           sndGrpHeight); */

/*    qgLevel->setGeometry (BUTTONWIDTH*3 + SPACE*9, 
                          SPACE,
                          (w - DEFAULTWIDTH)/2 + BUTTONWIDTH*2,
                          h - DEFAULTHEIGHT + BUTTONHEIGHT + 1); */

    mixerButton->setGeometry (((w - DEFAULTWIDTH)/2) + BUTTONWIDTH*5 + SPACE*9,
                              SPACE,
                              SPACE,
                              h - DEFAULTHEIGHT + BUTTONHEIGHT + 1);
  									
/*    qgFFT->setGeometry (((w - DEFAULTWIDTH)/2) + BUTTONWIDTH*5 + SPACE*10,
                        SPACE,
                        (w - DEFAULTWIDTH)/2 + BUTTONWIDTH*2,
                        h - DEFAULTHEIGHT + BUTTONHEIGHT + 1); */

    mp3recorder->resizeFrames ();
}


void KontinuidadJabata::create_statusline()
{
    statusline = this->statusBar ();
    statusline->insertItem ("C.A.Sonido Actual: 0:00:00", STAT_SNDCOUNTDOWN, 1);
    statusline->insertItem ("                          ", STAT_SEPARATOR, 1);
    statusline->insertItem ("C.A.Bloque Actual: 0:00:00", STAT_BLKCOUNTDOWN, 1);
    statusline->insertItem (" 00:00:00", STAT_CLOCK, 1);
}


void
KontinuidadJabata::refreshCurrentSound() {

  if(status==RECORDING || status==STOPINGRECORD) {
    qgCSound->setTitle(_("SONIDO ACTUAL (GRABANDO...)"));
    stopNowButton->setPixmap(recStopIcon);
    stopNowButton->show();
  }
  else {
    qgCSound->setTitle(_("SONIDO ACTUAL"));
    if((currentSound->hasSound()) && !(currentSound->isFESTIVAL())) {
      stopNowButton->setPixmap(stopIcon);
      stopNowButton->show();
    }
    else
      stopNowButton->hide();
  }

  lbCSndName->setText(currentSound->getName());
  lbCSndName->setMinimumSize(lbCSndName->sizeHint());
  lbCSndDir->setText(currentSound->getDirectory());
  lbCSndDir->setMinimumSize(lbCSndDir->sizeHint());
  lbCSndSubdir->setText(currentSound->getSubdirectory());
  lbCSndSubdir->setMinimumSize(lbCSndSubdir->sizeHint());
  lbCSndPath->setText(currentSound->getPathToSub());
  lbCSndPath->setMinimumSize(lbCSndPath->sizeHint());

  if(currentSound->hasSound()) {
	  sprintf(text, "%d:%02d:%02d", currentSound->getHours(),
  	                              currentSound->getMinutes(),
    	                            currentSound->getSeconds());
  	lbCSndTime->setText(text);
  	lbCSndTime->setMinimumSize(lbCSndTime->sizeHint());
    lbCSndTime->show();
  }
  else
    lbCSndTime->hide();

}

void
KontinuidadJabata::refreshNextSound() {
  fprintf(stderr, "KontinuidadJabata::refreshNextSound() Status:%d\n",status);

  lbSndName->setText(nextSound->getName());
  lbSndName->setMinimumSize(lbSndName->sizeHint());
  lbSndDir->setText(nextSound->getDirectory());
  lbSndDir->setMinimumSize(lbSndDir->sizeHint());
  lbSndSubdir->setText(nextSound->getSubdirectory());
  lbSndSubdir->setMinimumSize(lbSndSubdir->sizeHint());
  lbSndPath->setText(nextSound->getPathToSub());
  lbSndPath->setMinimumSize(lbSndPath->sizeHint());

  if(nextSound->hasSound()) {
    sprintf(text, "%d:%02d:%02d", nextSound->getHours(),
                              nextSound->getMinutes(),
                              nextSound->getSeconds());
    lbSndTime->setText(text);
    lbSndTime->setMinimumSize(lbSndTime->sizeHint());
    lbSndTime->show();
    if(recordPending)
      qgSound->setTitle(_("SIGUIENTE SONIDO... SE GRABARA"));
    else
      qgSound->setTitle(_("SIGUIENTE SONIDO (INTENTARA SONAR) "));
  }
  else {
    lbSndTime->hide();
  }

  if(kjabatamode!=3 && kjabatamode!=1 &&
     ((nextSound->isProgram() && !nextSound->isURL()) ||
      (nextSound->isSlot() && !nextSound->isFESTIVAL()))) {
    recordButton->show();
    if(kjabatamode==4)
      onAirCheck->show();
  }
  else {
    recordButton->hide();
    if(kjabatamode==4)
      onAirCheck->hide();
    recordPending = false;
  }
}


void
KontinuidadJabata::refreshCurrentBlock() {

  cbBlocks->setCurrentItem(filefeeder->blockIndex());
//  cbBlocks->setMinimumSize(cbBlocks->sizeHint());

  if(filefeeder->blockProgram())
    sprintf(text, "1 programa, %d temas y %d cuñas",
            filefeeder->blockSongs(),filefeeder->blockSlots());
  else
    sprintf(text, "%d temas y %d cuñas",
            filefeeder->blockSongs(),filefeeder->blockSlots());

  if(filefeeder->blockSlots()>0)
    sprintf(text, "%s (1 cuña%s cada %d temas%s)", text,
            (filefeeder->blockSlotsShuffled()?" aleatoria":""),
            filefeeder->blockSongsPerSlot(),
            (filefeeder->blockSongsShuffled()?" aleatorios":""));
  else
    sprintf(text, "%s (orden %s)", text,
             (filefeeder->blockSongsShuffled()?"aleatorio":"secuencial"));

  lbBlkSonidos->setText(_(text));
  lbBlkSonidos->setMinimumSize(lbBlkSonidos->sizeHint());

//  lbBlkOrden->setText(text);
//  lbBlkOrden->setMinimumSize(lbBlkOrden->sizeHint());

  sprintf(text, "%02d:%02d",
          filefeeder->blockBeginAtHour(),
          filefeeder->blockBeginAtMinute());
  lbBlkBegin->setText(text);
  lbBlkBegin->setMinimumSize(lbBlkBegin->sizeHint());

  sprintf(text, "%02d:%02d",
          filefeeder->blockEndAtHour(),
          filefeeder->blockEndAtMinute());
  lbBlkEnd->setText(text);
  lbBlkEnd->setMinimumSize(lbBlkEnd->sizeHint());

}


void
KontinuidadJabata::refreshTimers() {
  int h=hour, m=minute, s=second;
  int hh=0, mm=0, ss=0;
  int secsLeft;
  

  sprintf((char *)&text, "%02d:%02d:%02d", h, m, s);
  statusline->changeItem(text,STAT_CLOCK);

  if((status==STOPED) ||
     ((status==RECORDING || status==STOPINGRECORD) && changingBlock))
    lbBlkBeganAt->setText(text);

  if(status!=STOPED) {
    if(currentSound->timerStarted())
      currentSound->getElapsedTime(&hh,&mm,&ss);

    if(status==RECORDING || status==STOPINGRECORD) {
      if(!currentSound->timerStarted()) { 
        clksWaitingToRecord++;
        if(clksWaitingToRecord > WAITTORECORDCLKS) {
          clksWaitingToRecord = 0;
          mp3recorder->finish();
	}
	else
          sprintf(text, "%d:%02d:%02d %s", hh,mm,
	      WAITTORECORDCLKS-clksWaitingToRecord, 
    "                                                (ESPERANDO COMIENZO)");
      }
      else
        sprintf(text, "%d:%02d:%02d", hh,mm,ss);
	
      lbCSndTime->setText(text);
      lbCSndTime->setMinimumSize(lbCSndTime->sizeHint());
    }
    else {
      sprintf(text, "%d:%02d:%02d (%d:%02d:%02d)",
		    currentSound->getHours(),
	  	    currentSound->getMinutes(),
		    currentSound->getSeconds(),hh,mm,ss);
      lbCSndTime->setText(text);
      lbCSndTime->setMinimumSize(lbCSndTime->sizeHint());
    }
    
    currentSound->getLeftTime(&hh,&mm,&ss);
    sprintf((char *)&text, "C.A.Sonido Actual: %d:%02d:%02d", hh,mm,ss);
    statusline->changeItem(text,STAT_SNDCOUNTDOWN);

    if(changingBlock){
        sprintf((char *)&text, "C.A.Bloque Actual: %d:%02d:%02d", hh, mm, ss);
        statusline->changeItem(text,STAT_BLKCOUNTDOWN);
        s+=ss;
        m+=mm + s/60;
        s%=60;
        h+=hh + m/60;
        m%=60;
        sprintf((char *)&text, "%02d:%02d:%02d", h, m, s);
        lbBlkBeganAt->setText(text);
    }
  }

  if(kjabatamode!=1) {
    filefeeder->blockLeftTime(&h,&m,&s);
    if(nextSound->hasSound()) {
      if(kjabatamode==0 && 
         (status==RECORDING || status==STOPINGRECORD)) {
        if((!changingBlock && 
	    //!nextSound->isProgram() &&
            nextSound->getTimeInSeconds()>FORWARDSECONDS)
          ||
          (toSecs(h,m,s) < FORWARDSECONDS))
        lookforNextSound();
      }
      else {
        secsLeft = secsLeftByNextSound(h, m, s);
	
        if(schedule->autoRecMode() 
           && (kjabatamode==2 || kjabatamode==4) && status==STOPED) 
        {
          
	  if((h+m+s)==0 || toSecs(h,m,s)>86000) {	
            fprintf(stderr, 
		  "kontinuidadJabata:refreshTimers %d:%d:%d \n",h,m,s);
	    lookforNextSound();
          }
	}
	else if(secsLeft < -ERRORSECONDS)
        {
          if(nextSound->isProgram()) {
            if(secsLeft < -MAXPROGRAMDELAY)
              lookforNextSound();
          }
          else
            lookforNextSound();
        }
	
      }
    }
  }

  if((status==RECORDING || status==STOPINGRECORD) && ((hh+mm+ss)<1))
    mp3recorder->finish();
  
  if(!changingBlock) { 
    sprintf((char *)&text, "C.A.Bloque: %d:%02d:%02d", h, m, s);
    statusline->changeItem(text,STAT_BLKCOUNTDOWN);
    secsLeft=toSecs(h,m,s);
  }
  else
    secsLeft=MINSECSLEFTAUTOREC+1; 
  
  if(status==STOPED &&
     (kjabatamode==0||kjabatamode==2||kjabatamode==4) &&
     schedule->autoRecMode() &&  
     schedule->getCurrentStrip()->recordingStrip &&
     nextSound->isProgram() &&
     !autoRecorded) {
    if(secsLeft<MINSECSLEFTAUTOREC) // Un cuarto de hora antes del final dejamos
      autoRecorded=true;  // de intentar la grabación
    else {
      if(kjabatamode==4)
	onAirCheck->setChecked(true);
      autoRecording=true;
      record();
    }
  }
}

void
KontinuidadJabata::nextToCurrent() {
  JabatoSound *aux = currentSound;
  currentSound = nextSound;
  nextSound = aux;

  if(changingBlock) {
    int h,m,s;
    //qgBlock->setAlignment(AlignLeft);
    qgBlock->setTitle(_("BLOQUE ACTUAL"));
    lbBeganArrow->setText(">------------------------------------>");
    lbBeganArrow->show();
    filefeeder->systemClock(&h, &m, &s);
    sprintf((char *)&text, "%02d:%02d:%02d", h, m, s);
    lbBlkBeganAt->setText(text);
    changingBlock = false;
  }

}


void
KontinuidadJabata::playNextSound() {
  //DEBUG(fprintf(stderr, "kontinuidadJabata::playNextSound() \n"));
  fprintf(stderr, "kontinuidadJabata::playNextSound() \n");
  
  nextToCurrent();

  if(mp3player->play(currentSound)) {
    currentSound->startTimer();
    refreshCurrentSound();
    if(kjabatamode!=2 && kjabatamode!=4)
      lookforNextSound();
    else {
      nextSound->clear();
      refreshNextSound();
      status = STOPINGPLAY;
    }
  }
  else {
    QMessageBox::about(this, _("sorry..."),
		       _("problemas para lanzar el reproductor de sonidos"));
    nextSound->clear();
    status = STOPINGPLAY;
    playbackFinished(mp3player);
  }
}


void
KontinuidadJabata::recordNextSound() {
  DEBUG(fprintf(stderr, "kontinuidadJabata::recordNextSound() \n"));

  nextToCurrent();

  if((status!=STOPED)||(currentSound->isProgram()))
    lookforNextSound();
  else
    nextSound->clear();

  recordCurrentSound();

  refreshCurrentSound();
  refreshNextSound();
}


void
KontinuidadJabata::recordCurrentSound() {
  DEBUG(fprintf(stderr, "kontinuidadJabata::recordCurrentSound() \n"));

  if(mp3recorder->start(currentSound,
                        schedule->getStationName(),
                        replaceAcutes(filefeeder->blockJabatos()))) {
    int maxSecs;
    recorderTries=0;
    clksWaitingToRecord=0;
    currentSound->stopTimer();
    if(currentSound->isProgram()) {
      int h,m,s;
      maxSecs = filefeeder->blockLeftTime(&h,&m,&s);
      if(!autoRecording) {
	// CUANDO SE ANTICIPA EL COMIENZO AUTOMATICO DE UNA GRABACION,
	// EL TIEMPO RESTANTE DE BLOQUE (blockLeftTime) NOS DEVUELVE
	// LO QUE LE QUEDA AL BLOQUE ACTUAL, NO AL QUE ENTRA. MAL ASUNTO.
	// MIENTRAS LO INVESTIGO METO ESTA CHAPUCILLA. USO FORWARDSECONDS
	// POR USAR UNA CONSTANTE, SE PUEDE SUBSTITUIR PREFESTAMENTE.
	if(maxSecs<FORWARDSECONDS) 
	  maxSecs+=MAXPROGRAMDELAY*2;
	// FIN DE LA CHAPU.
        maxSecs+=MAXPROGRAMDELAY;
      }
      else
	autoRecording=false;
    }
    else {
      maxSecs = MAXSLOTSECS;
    }
    currentSound->setTimeInSeconds(maxSecs); 
    lbBlkllFinishAt->hide();
    lbFinishArrow->hide();
    qgFFT->show();
    qgLevel->show();
     
    if(status==STOPED) {
      nextSound->clear();
      status=STOPINGRECORD;
    }
    else
      status=RECORDING;

  }
  else {
    if(recorderTries>10) {
      recorderTries=0;
      QMessageBox::about(this, _("sorry..."),
   	       _("problemas para lanzar la grabadora de sonidos"));
      nextSound->clear();
      recordingFinished(mp3recorder);
    }
    else
      recorderTries++;
  }
}


void
KontinuidadJabata::lookforNextSound(int blockIndex) {
static bool alreadyLooking=false;

  DEBUG(fprintf(stderr, "kontinuidadJabata::lookforNextSound() %s\n",
                        alreadyLooking?"PROCESANDO ANTERIOR SOLICITUD!!!":""));
  if(!alreadyLooking) {
    int h,m,s,secsleft;

    alreadyLooking = true;
    if((status==STOPED) || (status==RECORDING) || (status==STOPINGRECORD))
      filefeeder->updateSound(nextSound,
                              FORWARDSECONDS,
                              blockIndex);
    else
      filefeeder->updateSound(nextSound,
                              currentSound->getLeftTimeInSeconds() + ERRORSECONDS,
                              blockIndex);

    refreshNextSound();

    if (filefeeder->blockUpdated()) {
      DEBUG(fprintf(stderr, "---------------------------------------> EL BLOQUE HA SIDO CAMBIADO \n"));
      changingBlock = true;
      autoRecorded=false;
      qgBlock->setTitle(_("SIGUIENTE BLOQUE"));
      lbBeganArrow->setText(">------------ comenzara a las... ----------->");
      refreshCurrentBlock();
      if(status==RECORDING || status==STOPINGRECORD)
        statusline->changeItem("C.A.Bloque: ?:??:??",STAT_BLKCOUNTDOWN);
      cbSlots->clear();
      for(int i=0;i<filefeeder->blockSlots();i++)
        cbSlots->insertItem(filefeeder->blockSlotAt(i));
      cbSlots->setMinimumSize(cbSlots->sizeHint());

    }

    if(status!=RECORDING && status!=STOPINGRECORD) {
      filefeeder->blockLeftTime(&h,&m,&s);
      secsleft = secsLeftByNextSound(h, m, s);
      DEBUG(fprintf(stderr, "EL S. AL S. SONIDO TIENE %d sg -->", secsleft));
      if(secsleft < ERRORSECONDS) {
        int s = filefeeder->blockEndAtHour()*3600 +
                filefeeder->blockEndAtMinute()*60 - secsleft;
        m = s/60;
        s%=60;
        h = m/60;
        m%=60;
        sprintf((char *)&text, "%02d:%02d:%02d", h, m, s);
        lbBlkllFinishAt->setText(text);
        lbBlkllFinishAt->show();
        lbFinishArrow->show();
        DEBUG(fprintf(stderr, "ULTIMO SONIDO DEL BLOQUE? \n"));
      }
      else {
        DEBUG(fprintf(stderr, "HABRÍA OTRO SONIDO MAS \n"));
        lbBlkllFinishAt->hide();
        lbFinishArrow->hide();
      }
    }
    alreadyLooking = false;
  }
}


int
KontinuidadJabata::toSecs(int h, int m, int s) {
  return (h*3600 + m*60 + s);
}


int
KontinuidadJabata::secsLeftByNextSound(int lh, int lm, int ls) {
  int dif = lh*3600 + lm*60 + ls - nextSound->getTimeInSeconds();
  if(status==PLAYING)
    dif -= currentSound->getLeftTimeInSeconds();

  return dif;
}

// *************** button signals *************** //
void KontinuidadJabata::playORstop ()
{
    bool justplayed=false;
    fprintf(stderr, "KontinuidadJabata::playORstop () status=%d\n", status);

    if ((status==STOPED) || (status==STOPINGPLAY) || (status==STOPINGRECORD))
    {
        if (status==STOPED) 
        {
            playNextSound();
            justplayed=true;
        }
        else
            lookforNextSound();

        if ((kjabatamode != KJABATA_RECORD_MODE && 
             kjabatamode != KJABATA_CHECKBOX_MODE) || 
            (status == STOPINGPLAY && !justplayed)) 
        {
            if(status==STOPINGRECORD)
                status = RECORDING;
            else
                status = PLAYING;
        }
    }
    else 
    {
        nextSound->clear();
        refreshNextSound();

        if(status==RECORDING)
            status = STOPINGRECORD;
        else
            status = STOPINGPLAY;
    }
}


void KontinuidadJabata::forwardSound() 
{
    if(status==STOPINGPLAY || status==STOPINGRECORD)
        playORstop();
    else
        lookforNextSound();
}


void KontinuidadJabata::record ()
{
    if (status==STOPED) 
    {
        recordNextSound ();
        qgSound->setTitle (_("SIGUIENTE SONIDO"));
    }
    else 
    {
        if (recordPending) 
        {
            forwardButton->show ();
            recordButton->setPixmap (recIcon);

            qgSound->setTitle (_("SIGUIENTE SONIDO (INTENTARA SONAR)"));
        }
        else 
        {
            forwardButton->hide ();
            recordButton->setPixmap (recStopIcon);
            qgSound->setTitle (_("SIGUIENTE SONIDO... SE GRABARA "));
        }

        recordPending = !recordPending;
    }

    if (startRecScript[0] && kjabatamode != KJABATA_CHECKBOX_MODE) 
    {
        shellProcess->clearArguments ();
        *shellProcess << startRecScript;
        shellProcess->start (KProcess::DontCare,  KProcess::NoCommunication);
        fprintf (stderr, 
	             "kontinuidadJabata::recordCurrentSound() Start script:%s \n",
                 startRecScript);
    }
}


void KontinuidadJabata::stopNow ()
{
    if(status==RECORDING || status==STOPINGRECORD) 
    {
        mp3recorder->finish();
        autoRecorded=true;
    }
    else
        mp3player->stop();
}


void
KontinuidadJabata::recordingStarted(MP3Recorder *rec) {
  fprintf(stderr, "KontinuidadJabata::recordingStarted()\n");
  currentSound->startTimer();
  clksWaitingToRecord = 0;
  if(kjabatamode==4 && onAirCheck->isChecked() && startRecScript[0] ) {
      shellProcess->clearArguments();
      *shellProcess << startRecScript;
      shellProcess->start(KProcess::DontCare,  KProcess::NoCommunication);
      fprintf(stderr, 
	      "kontinuidadJabata::recordCurrentSound() Start script:%s \n",
	      startRecScript);
  }
}

void
KontinuidadJabata::recordingHold(MP3Recorder *rec) {
  fprintf(stderr, "KontinuidadJabata::recordingHold()\n");
  //TODO: currentSound->elapsed();
  if(kjabatamode==4 && onAirCheck->isChecked() && holdRecScript[0]) {
    shellProcess->clearArguments();
    *shellProcess << holdRecScript;
    fprintf(stderr, 
       "kontinuidadJabata::recordingHold() Hold script: %s\n",
       holdRecScript);
    if(shellProcess->start(KProcess::DontCare,  KProcess::NoCommunication))
      fprintf(stderr, "%s OK!\n", holdRecScript);
    else
      fprintf(stderr, "%s ERROR!\n", holdRecScript);
  }
}

void
KontinuidadJabata::recordingRestarted(MP3Recorder *rec) {
  fprintf(stderr, "KontinuidadJabata::recordingRestarted()\n");
  //TODO: currentSound->elapsed();
  if(kjabatamode==4 && onAirCheck->isChecked() && restartRecScript[0]) {
    shellProcess->clearArguments();
    *shellProcess << restartRecScript;
    shellProcess->start(KProcess::DontCare,  KProcess::NoCommunication);
    fprintf(stderr, 
       "kontinuidadJabata::recordingRestarted() Restart script: %s\n",
       restartRecScript);
  }
}


		    

void
KontinuidadJabata::recordingFinished(MP3Recorder *rec) {

  fprintf(stderr, "KontinuidadJabata::recordingFinished()\n");
  sprintf(programsPath, "%s/%s", currentSound->getPathToSub(),
                                 currentSound->getSubdirectory());
  freeDiskProcess->clearArguments();
  *freeDiskProcess << "df -k " << programsPath << "| grep ^/ | awk '{ print $4}'";
  freeDiskProcess->start(KProcess::DontCare, KProcess::Stdout);
  if(currentSound->execBackupCommands() && //GRABADO SONIDO VALIDO!!
    schedule->autoRecMode())
    autoRecorded=true;

  if(recordPending) {
    recordPending = false;
    recordNextSound();
    if(kjabatamode!=1) forwardButton->show();
    recordButton->setPixmap(recIcon);
  }
  else {
    mixerButton->hide();
    qgFFT->hide();
    qgLevel->hide();
    if(nextSound->hasSound()) {
      status=PLAYING;
      playNextSound();
    }
    else {
      status=STOPED;
      JabatoSound *aux = nextSound;
      currentSound->stopTimer();
      currentSound->updateTime();
      nextSound = currentSound;
      currentSound = aux;
      refreshNextSound();
      refreshCurrentSound();
    }
  }

  if(kjabatamode==4 && onAirCheck->isChecked() && finishRecScript[0]) {
    shellProcess->clearArguments();
    *shellProcess << finishRecScript;
    fprintf(stderr, 
       "kontinuidadJabata::recordingFinished() Finish script:%s \n",
       finishRecScript);
    if(shellProcess->start(KProcess::DontCare,  KProcess::NoCommunication))
      fprintf(stderr, "%s OK!\n", finishRecScript);
    else
      fprintf(stderr, "%s ERROR!\n", finishRecScript);
    
  }
  
}


void
KontinuidadJabata::playbackFinished(MP3Player *pl)
{
  DEBUG(fprintf(stderr, "KontinuidadJabata::playbackFinished()\n"));
  if(recordPending) {
    recordNextSound();
    recordPending = false;
    if(kjabatamode!=1) forwardButton->show();
    recordButton->setPixmap(recIcon);
  }
  else {
    if (status==PLAYING)
      playNextSound();
    else {
      status = STOPED;
      currentSound->stopTimer();
      JabatoSound *aux = nextSound;
      nextSound = currentSound;
      currentSound = aux;
      currentSound->clear();
      refreshNextSound();
      refreshCurrentSound();
    }
  }
}


void
KontinuidadJabata::timeout(){
static int flag=0;
/* THIS IS THE ACTUAL HEART OF THE APPLICATION, THE TIMER */

  if(recorderTries) {
    recordCurrentSound();
    refreshCurrentSound();
  }
  else {
    ++flag%=10;
    if(recordPending || changingBlock) {
      if(flag%2) {
      	if(recordPending) {
          recordButton->setPixmap(0);
	  qgSound->setTitle(_("SIGUIENTE SONIDO...                      "));
      	}
	if(changingBlock) {
	  lbBeganArrow->hide();
	}
      }
      else {
        if(recordPending) {
          recordButton->setPixmap(recStopIcon);
  	  qgSound->setTitle(_("SIGUIENTE SONIDO... SE GRABARA "));
	}
  	if(changingBlock)
	   lbBeganArrow->show();
      }
    }

    dow = filefeeder->systemClock(&hour, &minute, &second);
    refreshTimers();

    if(kjabatamode!=2 && kjabatamode!=4 && flag==0) {
      schedule->checkTasks(dow,hour,minute);
    }
  }

  QTimer::singleShot(CLOCKMILISECONDS, this, SLOT(timeout()));
}


void
KontinuidadJabata::blockChanged(int index){
  fprintf(stderr, "KontinuidadJabata::blockChanged() Status:%d\n",status);
  if(status==STOPINGPLAY || status==STOPINGRECORD) {
    if(status==STOPINGPLAY)
      status = PLAYING;
    else
      status = RECORDING;
  }
  lookforNextSound(index);
  autoRecorded=true;
}

void
KontinuidadJabata::slotChanged(int index){
  fprintf(stderr, "KontinuidadJabata::slotChanged() Status:%d\n",status);
  if(status==STOPINGPLAY || status==STOPINGRECORD) {
    if(status==STOPINGPLAY)
      status = PLAYING;
    else
      status = RECORDING;
  }
  nextSound->setFullPath(cbSlots->text(index));
  nextSound->setSoundType(JABATOSLOT);
  refreshNextSound();
}


void
KontinuidadJabata::freeDiskOutput(KProcess *proc,char *buffer, int len) {
  long freebytes = atol(buffer);

  fprintf(stderr, "kjabata::freeDiskOutput: free bytes %d (min. %d)\n", 
		  (int)freebytes,(int)minFreeDiskInBytes);
  if(freebytes < minFreeDiskInBytes) {
    fprintf(stderr, "\nMIN FREE DISK REACHED: Removing... ");
    //    RECORDERPROGRAMSDIR --> jabatoSound.h
    QDir d(RECORDERPROGRAMSDIR,"2*",QDir::Time|QDir::Reversed);
    //                          ^
    //                          |
    //             NO PREPARADO PARA EL AÑO 3000 :(
    if(d.exists()&&d.isReadable())
    {
      const QFileInfoList *list = d.entryInfoList();
      QFileInfoListIterator it( *list );
      QFileInfo  *filename;
      
      if((filename=it.current())) {
        if(d.remove(d.absFilePath(filename->fileName()).data()))
       	  fprintf(stderr, "%s REMOVED\n", filename->fileName().data());
	else
      	  fprintf(stderr, "ERROR: CANNOT REMOVE %s!!!\n", 
			  filename->fileName().data());
      }

/*      QStrListIterator it(&(d.entryList()));
      if((filename=it.current())) {
        if(d.remove(d.absFilePath(filename).data()))
       	  fprintf(stderr, "%s REMOVED\n", filename);
	else
      	  fprintf(stderr, "ERROR: CANNOT REMOVE %s!!!\n", filename);
      } */
    }
  }
}


void
KontinuidadJabata::runMixer()
{
  if (!fork()) {
	  execlp("kmix","kmix",NULL);
	  exit(0);
  }
}


